(define-module (alterator gettext)
  :use-module (srfi srfi-1)
  :use-module (srfi srfi-13)
  :use-module (alterator algo)
  :use-module (alterator str)
  :export (make-translator
           make-translator-cache
           translator-cache-ref
           translate
           default-po-domain))

(dynamic-call "scm_init_gettext"
	      (dynamic-link "libguile-gettext.so"))

;;create path like /usr/share/locale/ru/LC_MESSAGE/package.mo
(define (make-system-path item package)
  (string-append "/usr/share/locale/" item "/LC_MESSAGES/" package ".mo"))

(define (search-mo package candidates)
  (find read-access?
        (map (lambda (x) (make-system-path x package))
              (fold (lambda (x y) ;;append short names to expand searching
		      (if (> (string-length x) 2)
			(append y (list x (string-take x 2)))
			y))
                    '()
                    candidates))))

(define (load-dictionary mo-path)
  (let ((dict (make-hash-table 10)))
    (for-each (lambda (item)
                (hash-set! dict (car item) (cdr item)))
              (or (load-mo mo-path) '()))
    dict))

(define (search-dictionary word dict)
  (if dict (hash-ref dict word word) word))

(define (make-dictionary package candidates)
  (let ((full-path (search-mo package candidates)))
       (and full-path (load-dictionary full-path))))

(define (make-translator2 dict)
  (lambda(str) (search-dictionary str dict)))

(define (make-translator package candidates)
  (make-translator2 (make-dictionary package candidates)))

;; translator-cash

(define (make-translator-cache candidates)
  (cons (make-hash-table 10) candidates))

(define (translator-cache-ref hsh domain)
  (let ((tr (hash-ref (car hsh) domain)))
    (or tr
        (let ((tr (make-translator domain (cdr hsh))))
          (hash-set! (car hsh) domain tr)
          tr))))

(define (translate hsh str domain default-domain)
  (if (string-null? str)
    str
    (let* ((d (if (string=? domain "_") default-domain domain))
	   (tr (translator-cache-ref hsh d)))
      (tr str))))

(define (default-po-domain url)
  (let* ((url (string-trim-both url #\/))
	 (start (and (not (string-null? url)) (string-index url #\/ 0)))
	 (base (substring url 0 (or start (string-length url)))))
    (string-append "alterator-" base)))
